/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/time.h>

#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/human/shapes.h>

namespace armarx::navigation::human
{
    struct Human
    {
        core::Pose2D pose;
        Eigen::Vector2f linearVelocity;
        DateTime detectionTime;

        core::Pose2D estimateAt(const DateTime& time) const;

        aron::data::DictPtr toAron() const;
        static Human FromAron(const aron::data::DictPtr& dict);
    };

    using Humans = std::vector<Human>;


    struct HumanGroup
    {
        shapes::Polygon shape;
        Humans humans;
        DateTime detectionTime;

        aron::data::DictPtr toAron() const;
        static HumanGroup FromAron(const aron::data::DictPtr& dict);
    };

    using HumanGroups = std::vector<HumanGroup>;

    struct LinearPenaltyModel
    {
        float minDistance; // [m]
        float epsilon; // [m]
    };

    struct ExponentialPenaltyModel
    {
        float minDistance; // [m]
        float epsilon; // [m]
        float exponent;
    };

    struct ProxemicZone
    {
        core::Pose2D pose;
        shapes::Ellipse shape;
        ExponentialPenaltyModel penalty;
        float weight;
        bool homotopicRelevance = true;
    };

    using ProxemicZones = std::vector<ProxemicZone>;

} // namespace armarx::navigation::human
